@@ -25,15 +25,38 @@ Hash::SharedMem - efficient shared mutable hash
 	$mode = shash_mode($shash);
 
 	use Hash::SharedMem qw(
-		shash_getd shash_get shash_set shash_gset shash_cset
+		shash_exists shash_length shash_get
+		shash_set shash_gset shash_cset
 	);
 
-	if(shash_getd($shash, $key)) { ...
+	if(shash_exists($shash, $key)) { ...
+	$length = shash_length($shash, $key);
 	$value = shash_get($shash, $key);
 	shash_set($shash, $key, $newvalue);
 	$oldvalue = shash_gset($shash, $key, $newvalue);
 	if(shash_cset($shash, $key, $chkvalue, $newvalue)) { ...
 
+	use Hash::SharedMem qw(
+		shash_occupied shash_count shash_size
+		shash_key_min shash_key_max
+		shash_key_ge shash_key_gt shash_key_le shash_key_lt
+		shash_keys_array shash_keys_hash
+		shash_group_get_hash
+	);
+
+	if(shash_occupied($shash)) { ...
+	$count = shash_count($shash);
+	$size = shash_size($shash);
+	$key = shash_key_min($shash);
+	$key = shash_key_max($shash);
+	$key = shash_key_ge($shash, $key);
+	$key = shash_key_gt($shash, $key);
+	$key = shash_key_le($shash, $key);
+	$key = shash_key_lt($shash, $key);
+	$keys = shash_keys_array($shash);
+	$keys = shash_keys_hash($shash);
+	$group = shash_group_get_hash($shash);
+
 	use Hash::SharedMem qw(shash_snapshot shash_is_snapshot);
 
 	$snap_shash = shash_snapshot($shash);
@@ -306,14 +329,17 @@ A shared hash handle per se is never tainted.
 
 The results of the mode checking functions are not tainted.
 
-It is permitted to write tainted data to a shared hash.  The data
-operations all accept tainted arguments.  When reading from a shared hash,
-the value referenced by any key existing in the hash is always tainted,
-but an absent value is treated as clean.  So where a data operation
-returns a value from the shared hash, the value will be tainted if it is
-a string, but C<undef> representing an absent value will not be tainted.
-The truth values returned by some operations are not tainted, even if
-they are derived entirely from tainted data.
+The content of a shared hash is always treated as tainted.  It is
+permitted to write tainted data to a shared hash.  The data operations
+all accept tainted arguments.  When reading from a shared hash, the keys
+existing in the hash and the values referenced by them are always tainted,
+but an absent item is treated as clean.  So where a data operation returns
+a key or value from the shared hash, the result will be tainted if it is
+a string, but C<undef> representing an absent item will not be tainted.
+The count of keys existing in the hash, size of the hash, and the length
+of an existing value are also tainted, being derived from tainted content.
+However, the truth values returned by some operations are not tainted,
+even if they are derived entirely from tainted data.
 
 =cut
 
@@ -323,7 +349,7 @@ package Hash::SharedMem;
 use warnings;
 use strict;
 
-our $VERSION = "0.003";
+our $VERSION = "0.004";
 
 use parent "Exporter";
 our @EXPORT_OK = qw(
@@ -331,7 +357,13 @@ our @EXPORT_OK = qw(
 	is_shash check_shash
 	shash_open
 	shash_is_readable shash_is_writable shash_mode
-	shash_getd shash_get shash_set shash_gset shash_cset
+	shash_exists shash_getd shash_length shash_get
+	shash_set shash_gset shash_cset
+	shash_occupied shash_count shash_size
+	shash_key_min shash_key_max
+	shash_key_ge shash_key_gt shash_key_le shash_key_lt
+	shash_keys_array shash_keys_hash
+	shash_group_get_hash
 	shash_snapshot shash_is_snapshot
 	shash_idle shash_tidy
 	shash_tally_get shash_tally_zero shash_tally_gzero
@@ -404,7 +436,7 @@ Returns normally if it is, or C<die>s if it is not.
 
 =item shash_open(FILENAME, MODE)
 
-Opens and return a handle referring to a shared hash object, or C<die>s
+Opens and returns a handle referring to a shared hash object, or C<die>s
 if the shared hash can't be opened as specified.  I<FILENAME> must
 refer to the directory that encapsulates the shared hash.
 
@@ -507,7 +539,7 @@ The shared hash can be written to through this handle.
 
 =back
 
-=head2 Data operations
+=head2 Single-key data operations
 
 For all of these functions, the key of interest (I<KEY> parameter)
 can be any octet (Latin-1) string, and values (I<VALUE> parameters and
@@ -520,11 +552,20 @@ values) is supplied, only its string value will be used.
 
 =over
 
-=item shash_getd(SHASH, KEY)
+=item shash_exists(SHASH, KEY)
 
 Returns a truth value indicating whether the specified key currently
 references a defined value in the shared hash.
 
+=item shash_getd(SHASH, KEY)
+
+Deprecated alias for L</shash_exists>.
+
+=item shash_length(SHASH, KEY)
+
+Returns the length (in octets) of the value currently referenced by the
+specified key in the shared hash, or C<undef> if the key is absent.
+
 =item shash_get(SHASH, KEY)
 
 Returns the value currently referenced by the specified key in the
@@ -561,6 +602,102 @@ an atomic increment can be implemented as
 
 =back
 
+=head2 Whole-hash data operations
+
+=over
+
+=item shash_occupied(SHASH)
+
+Returns a truth value indicating whether there are currently any items
+in the shared hash.
+
+=item shash_count(SHASH)
+
+Returns the number of items that are currently in the shared hash.
+
+=item shash_size(SHASH)
+
+Returns the approximate size (in octets) of the entire content of the
+shared hash.  The size of a hash is not a well-defined quantity, so the
+return value of this function should be interpreted with care.  It aims
+specifically to indicate how much space is required in a shared hash
+data file to represent this content.  It is affected by details of the
+file format (which may differ between shared hashes on one system) and
+by accidents of how the content is laid out in a particular shared hash.
+Calling this function twice on identical content will not necessarily
+produce identical results.  The details of the size estimation may also
+change in the future.
+
+Although this function computes size specifically with respect to the
+file format used by this module, this function does not directly indicate
+the amount of space occupied by a shared hash.  There is some non-content
+overhead, and, more importantly, the process by which content is modified
+requires space to store multiple versions of the content.  It is normal
+for the amount of space actually occupied to fluctuate over the cycle
+of data file rewriting.  If L</shash_tidy> is being used appropriately,
+the space occupied can be expected to vary up to a little over five
+times the size of the nominal content, and if L</shash_tidy> is not used
+then the normal maximum will be more than ten times the content size.
+Occasional spikes above these levels can be expected in any case, and
+fixed overheads make these multipliers inapplicable if the content is
+very small.
+
+=item shash_key_min(SHASH)
+
+Returns the lexicographically least of the keys that are currently in
+the shared hash, or C<undef> if there are none.
+
+=item shash_key_max(SHASH)
+
+Returns the lexicographically greatest of the keys that are currently
+in the shared hash, or C<undef> if there are none.
+
+=item shash_key_ge(SHASH, KEY)
+
+Returns the least of the keys currently in the shared hash that are
+lexicographically no less than the specified key, or C<undef> if there
+are none.
+
+=item shash_key_gt(SHASH, KEY)
+
+Returns the least of the keys currently in the shared hash that are
+lexicographically greater than the specified key, or C<undef> if there
+are none.
+
+=item shash_key_le(SHASH, KEY)
+
+Returns the greatest of the keys currently in the shared hash that
+are lexicographically no greater than the specified key, or C<undef>
+if there are none.
+
+=item shash_key_lt(SHASH, KEY)
+
+Returns the greatest of the keys currently in the shared hash that are
+lexicographically less than the specified key, or C<undef> if there
+are none.
+
+=item shash_keys_array(SHASH)
+
+Returns a reference to a plain array containing all the keys currently
+in the shared hash in lexicographical order.  The array and the key
+scalars in it are unwritable.
+
+=item shash_keys_hash(SHASH)
+
+Returns a reference to a plain hash in which the keys are all the keys
+currently in the shared hash and all the values are C<undef>.  The value
+scalars are unwritable.  Writability of the hash is not guaranteed:
+currently in practice it is writable, but this may change in the future.
+
+=item shash_group_get_hash(SHASH)
+
+Returns a reference to a plain hash representing the entire current
+content of the shared hash.  The value scalars are unwritable.
+Writability of the hash is not guaranteed: currently in practice it is
+writable, but this may change in the future.
+
+=back
+
 =head2 Snapshots
 
 =over
@@ -615,8 +752,10 @@ existence of an obsolete data file.
 Rearranges the storage of the shared hash if it seems useful to do
 so, to avoid tidying work having to be performed by other processes.
 This doesn't change the visible content of the shared hash, but the
-handle must be open for writing.  The invisible operations performed by
-this function may vary between versions of this module.
+handle must be open for writing, and this counts as a write operation for
+purposes concerned with the state of the underlying files.  The invisible
+operations performed by this function may vary between versions of
+this module.
 
 This function should be called in circumstances where it is acceptable
 to incur some delay for this maintenance work to complete.  For example,
@@ -714,7 +853,12 @@ initialising its new one.
 =item B<data_read_op>
 
 Perform a high-level data operation that purely reads from the shared
-hash: L</shash_getd> or L</shash_get>.
+hash: L</shash_exists>, L</shash_length>, L</shash_get>,
+L</shash_occupied>, L</shash_count>, L</shash_size>,
+L</shash_key_min>, L</shash_key_max>,
+L</shash_key_ge>, L</shash_key_gt>, L</shash_key_le>, L</shash_key_lt>,
+L</shash_keys_array>, L</shash_keys_hash>,
+or L</shash_group_get_hash>.
 
 =item B<data_write_op>
 
@@ -723,6 +867,10 @@ to the shared hash: L</shash_set>, L</shash_gset>, or L</shash_cset>.
 
 =back
 
+The value scalars in the returned hash are unwritable.  Writability of
+the hash is not guaranteed: currently in practice it is writable, but
+this may change in the future.
+
 =item shash_tally_zero(SHASH)
 
 Zero the event counters that can be read by L</shash_tally_get>.
@@ -764,9 +912,9 @@ Andrew Main (Zefram) <zefram@fysh.org>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2014 PhotoBox Ltd
+Copyright (C) 2014, 2015 PhotoBox Ltd
 
-Copyright (C) 2014 Andrew Main (Zefram) <zefram@fysh.org>
+Copyright (C) 2014, 2015 Andrew Main (Zefram) <zefram@fysh.org>
 
 =head1 LICENSE
 
